#!/bin/sh
##------------------------------------------------------------------------------
## (opt/files)/srv/www/admin/macfilter.cgi
##------------------------------------------------------------------------------
## DE: Diese Datei ist Teil von MACFilter einem OPT fr den fli4l Router.
##     Sehe config/macfilter.txt ein fr Kontakt- und Lizenzinformationen.
## EN: This file is part of MACFilter an opt for fli4l.
##     See config/macfilter.txt for contact and license information.
##------------------------------------------------------------------------------
## Creation:        01.11.2008  Florian Schlgl <Florian_Schloegl.Wolfsteam@web.de>
## Last modified:   28.07.2009  Florian Schlgl <Florian_Schloegl.Wolfsteam@web.de>
## Version:         2.0.0-2.0.5
##------------------------------------------------------------------------------
## DE: OPT_HTTPD Webadmin Webseite fr MACFilter
## EN: OPT_HTTPD webadmin page for MACFilter
##------------------------------------------------------------------------------
. /srv/www/include/cgi-helper
. /var/run/macfilter.conf
. /usr/local/bin/macfilter_log.include

macfilter_log_trace "general trace: cgi call (QUERY_STRING='$QUERY_STRING')"

#-------------------------------------------------------------------------------
# DE: HTML header ausgeben
# EN: print html header
#-------------------------------------------------------------------------------
show_html_header "$_html_header"

#-------------------------------------------------------------------------------
# DE: Aktion durchfhren
# EN: perform action
#-------------------------------------------------------------------------------
: ${FORM_action:=view}
case $FORM_action in
    #----------------------------------------
    # DE: Mac hinzufgen
    # EN: add mac
    #----------------------------------------
    'add')
        check_rights "macfilter" "add"
        case `/usr/local/bin/macfilter.sh "exist" "$FORM_mac"` in
            'not found')
                mf_retval=`/usr/local/bin/macfilter.sh "add" "$FORM_mac" "$FORM_comment"`
                if [ "$mf_retval" = "added" ]
                then
                    show_info "$_info_added_mac_header" "$_info_added_mac_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                    echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
                else
                    show_error "$_error_unexpected_returncode" "macfilter.sh add: '$mf_retval'"
                fi
            ;;
            'added')
                show_info "$_info_already_added_header" "$_info_already_added_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
            ;;
            'invalid mac')
                show_error "$_error_invalid_mac_header" "$_error_invalid_mac_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
            ;;
            *)
                show_error "$_error_unexpected_returncode" "macfilter.sh exist: '$mf_retval'"
            ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: Mac entfernen
    # EN:remove mac
    #----------------------------------------
    'remove')
        check_rights "macfilter" "remove"
        case `/usr/local/bin/macfilter.sh "exist" "$FORM_mac"` in
            'added')
                mf_retval=`/usr/local/bin/macfilter.sh "remove" "$FORM_mac"`
                if [ "$mf_retval" = "removed" ]
                then
                    show_info "$_info_removed_mac_header" "$_info_removed_mac_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                    echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
                else
                    show_error "$_error_unexpected_returncode" "macfilter.sh remove: '$mf_retval'"
                fi
            ;;
            'not found')
                show_info "$_info_mac_not_found_header" "$_info_mac_not_found_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
            ;;
            'invalid mac')
                show_error "$_error_invalid_mac_header" "$_error_invalid_mac_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
            ;;
            *)
                show_error "$_error_unexpected_returncode" "macfilter.sh exist: '$mf_retval'"
            ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: Macfilter starten
    # EN: start macfilter
    #----------------------------------------
    'start')
        check_rights "macfilter" "startfilter"
        case `/usr/local/bin/macfilter.sh "status"` in
            'stopped')
                mf_retval=`/usr/local/bin/macfilter.sh "start"`
                if [ "$mf_retval" = "started" ]
                then
                    show_info "$_info_starting_header" "$_info_starting_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                    echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
                else
                    show_error "$_error_unexpected_returncode" "macfilter.sh start: '$mf_retval'"
                fi
            ;;
            'started')
                show_info "$_info_already_running_header" "$_info_already_running_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
            ;;
            *)
                show_error "$_error_unexpected_returncode" "macfilter.sh status: '$mf_retval'"
            ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: Macfilter anhalten
    # EN: stop macfilter
    #----------------------------------------
    'stop')
        check_rights "macfilter" "stopfilter"
        case `/usr/local/bin/macfilter.sh "status"` in
            'started')
                mf_retval=`/usr/local/bin/macfilter.sh "stop"`
                if [ "$mf_retval" = "stopped" ]
                then
                    show_info "$_info_stopping_header" "$_info_stopping_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                    echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
                else
                    show_error "$_error_unexpected_returncode" "macfilter.sh stop: '$mf_retval'"
                fi
            ;;
            'stopped')
                show_info "$_info_not_running_header" "$_info_not_running_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
            ;;
            *)
                show_error "$_error_unexpected_returncode" "macfilter.sh status: '$mf_retval'"
            ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: Auf Whitelist stellen
    # EN: set to whitelist
    #----------------------------------------
    'whitelist')
        check_rights "macfilter" "whitelist"
        if [ `/usr/local/bin/macfilter.sh "status"` = "stopped" ]
        then
            check_rights "macfilter" "startfilter"
            mf_retval=`/usr/local/bin/macfilter.sh "start"`
            if [ "$mf_retval" = "started" ]
            then
                show_info "$_info_starting_header" "$_info_starting_text"
            else
                show_error "$_error_unexpected_returncode" "macfilter.sh start: '$mf_retval'"
            fi
            echo "<p/>"
        fi
        
        case `/usr/local/bin/macfilter.sh "listtype"` in
            'blacklist')
                mf_retval=`/usr/local/bin/macfilter.sh "whitelist"`
                if [ "$mf_retval" = "set to whitelist" ]
                then
                    show_info "$_info_whitelist_header" "$_info_whitelist_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                    echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
                else
                    show_error "$_error_unexpected_returncode" "macfilter.sh whitelist: '$mf_retval'"
                fi
            ;;
            'whitelist')
                show_info "$_info_already_whitelist_header" "$_info_already_whitelist_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
            ;;
            *)
                show_error "$_error_unexpected_returncode" "macfilter.sh listtype: '$mf_retval'"
            ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: Auf Blacklist stellen
    # EN: set to blacklist
    #----------------------------------------
    'blacklist')
        check_rights "macfilter" "whitelist"
        if [ `/usr/local/bin/macfilter.sh "status"` = "stopped" ]
        then
            check_rights "macfilter" "startfilter"
            mf_retval=`/usr/local/bin/macfilter.sh "start"`
            if [ "$mf_retval" = "started" ]
            then
                show_info "$_info_starting_header" "$_info_starting_text"
            else
                show_error "$_error_unexpected_returncode" "macfilter.sh start: '$mf_retval'"
            fi
            echo "<p/>"
        fi
        
        case `/usr/local/bin/macfilter.sh "listtype"` in
            'whitelist')
                mf_retval=`/usr/local/bin/macfilter.sh "blacklist"`
                if [ "$mf_retval" = "set to blacklist" ]
                then
                    show_info "$_info_blacklist_header" "$_info_blacklist_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                    echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
                else
                    show_error "$_error_unexpected_returncode" "macfilter.sh blacklist: '$mf_retval'"
                fi
            ;;
            'blacklist')
                show_info "$_info_already_blacklist_header" "$_info_already_blacklist_text<CENTER><A href=\"$myname\">$_cmd_go_back</A></CENTER>"
                echo "<META http-equiv=\"refresh\" content=\"3; URL=$myname\">"
            ;;
            *)
                show_error "$_error_unexpected_returncode" "macfilter.sh listtype: '$mf_retval'"
            ;;
        esac
    ;;

    #-------------------------------------------------------------------------------
    # DE: HTML ausgeben
    # EN: print HTML
    #-------------------------------------------------------------------------------
    'view')
        mf_status=`/usr/local/bin/macfilter.sh "status"`
        mf_blacklist_url="$myname?action=blacklist"
        mf_whitelist_url="$myname?action=whitelist"
        mf_stopped_url="$myname?action=stop"
        case $mf_status in
            'started')
                mf_listtype=`/usr/local/bin/macfilter.sh "listtype"`
                case $mf_listtype in
                    'blacklist')    mf_blacklist_url=no ;;
                    'whitelist')    mf_whitelist_url=no ;;
                    *)
                        macfilter_log_error "unexpected listtype returncode: '$mf_listtype'"
                        show_error "$_error_unexpected_returncode" "macfilter.sh listtype: '$mf_listtype'"
                    ;;
                esac
            ;;
            'stopped')      mf_stopped_url=no ;;
            *)
                macfilter_log_error "unexpected status returncode: '$mf_status'"
                show_error "$_error_unexpected_returncode" "macfilter.sh status: '$mf_status'"
            ;;
        esac
        show_tab_header  "$_tab_blacklist"  "$mf_blacklist_url" \
                         "$_tab_whitelist"  "$mf_whitelist_url" \
                         "$_tab_stop"       "$mf_stopped_url"
        
        ##-------------------------------------------------------------------------------
        ## DE: MAC Addressen Tabelle ausgeben
        ## EN: print MAC address table
        ##-------------------------------------------------------------------------------
        cat <<-EOF
<TABLE class="normtable">
  <TR>
    <TH align="left">$_table_header_mac</TH>
    <TH align="left">$_table_header_comment</TH>
    <TH align="left">$_table_header_action</TH>
  </TR>
EOF
        for mf_mac in `/usr/local/bin/macfilter.sh "list"`
        do
            mf_comment=`/usr/local/bin/macfilter.sh "comment" "$mf_mac"`
            cat <<-EOF
  <TR><FORM action="$myname" method="get">
    <INPUT type="hidden" name="action" value="remove">
    <INPUT type="hidden" name="mac" value="$mf_mac">
    <TD align="left">$mf_mac</TD>
    <TD align="left">$mf_comment</TD>
    <TD align="center">
      <INPUT type="submit" value="$_table_action_remove" class="actions">
    </TD>
  </FORM></TR>
EOF
        done
        
        cat <<-EOF
          <FORM action="$myname" method="get"><TR>
    <INPUT type="hidden" name="action" value="add">
    <TD align="left">
      <INPUT type="text" name="mac" value="">
    </TD>
    <TD align="left">
      <INPUT type="text" name="comment" value="">
    </TD>
    <TD align="center"><INPUT type="submit" value="$_table_action_add" class="actions"></TD>
  </TR></FORM>
</TABLE>
EOF
        show_tab_footer
    ;;
    
    #----------------------------------------
    # DE: unbekannte Aktion
    # EN: unknown action
    #----------------------------------------
    *)
        show_error "$_error_unknown_action_header" "$_error_unknown_action"
    ;;
esac

#----------------------------------------
# DE: Ende des Dokuments ausgeben
# EN: print end of doc
#----------------------------------------
show_html_footer
##------------------------------------------------------------------------------
# macfilter.cgi -- EOF ---
##------------------------------------------------------------------------------
