#!/bin/sh
##------------------------------------------------------------------------------
## (opt/files)/usr/local/bin/macfilter.sh
##------------------------------------------------------------------------------
## DE: Diese Datei ist Teil von MACFilter einem OPT fr den fli4l Router.
##     Sehe config/macfilter.txt ein fr Kontakt- und Lizenzinformationen.
## EN: This file is part of MACFilter an opt for fli4l.
##     See config/macfilter.txt for contact and license information.
##------------------------------------------------------------------------------
## Creation:        01.11.2008  Florian Schlgl <Florian_Schloegl.Wolfsteam@web.de>
## Last modified:   04.09.2009  Florian Schlgl <Florian_Schloegl.Wolfsteam@web.de>
## Version:         2.0.3-2.0.5
##------------------------------------------------------------------------------
## DE: start, stop, hinzufgen und entfernen von MACs
## EN: start, stop, add/remove of MACs
##------------------------------------------------------------------------------
## Parameter:
##      MACFilter starten
##          Call:   <$1 = "start">
##          Return: "started"
##      MACFilter anhalten
##          Call:   <$1 = "stop">
##          Return: "stopped"
##      Status des MACFilters ausgeben
##          Call:   <$1 = "status">
##          Return: "started", "stopped"
##
##      MACliste fr Blacklist benutzen
##          Call:   <$1 = "blacklist">
##          Return: "set to blacklist"
##      MACliste fr Whitelist benutzen
##          Call:   <$1 = "whitelist">
##          Return: "set to whitelist"
##      Status des MACFilters ausgeben
##          Call:   <$1 = "listtype">
##          Return: "blacklist", "whitelist"
##
##      MAC hinzufgen
##          Call:   <$1 = "add"> <$2 = MAC> <$3 = Comment>  
##          Return: "added", "invalid mac"
##      Kontrolle ob eine MAC in der Liste existiert
##          Call:   <$1 = "exist"> <$2 = MAC>
##          Return: "added", "not found", "invalid mac"
##      Kommentar einer MAC ausgeben
##          Call:   <$1 = "comment"> <$2 = MAC>
##          Return: "not found", "invalid mac", Kommentar der MAC
##      MAC entfernen
##          Call:   <$1 = "remove"> <$2 = MAC>
##          Return: "removed", "invalid mac"
##      Liste mit den MAC Adressen ausgeben
##          Call:   <$1 = "list">
##          Return: List mit MAC Adressen
##------------------------------------------------------------------------------
. /etc/boot.d/base-helper
. /etc/rc.d/fwrules-helper
. /srv/www/include/cgi-helper
. /var/run/macfilter.conf
. /usr/local/bin/macfilter_log.include

macfilter_log_trace "general trace: shell call"

##------------------------------------------------------------------------------
# DE: Gibt den Kommentar einer bergebenen iptables eintrags wieder
# EN: extracts the comment of a given iptables row
##------------------------------------------------------------------------------
createIPTablesComment() {
    mf_iptables_comment="MAC:'$1'; comment:'$2'"
}

macfilter_echo_log_error() {
    echo $@
    macfilter_log_error $@
}

macfilter_echo_log_warn() {
    echo $@
    macfilter_log_warn $@
}

macfilter_make_uppercase() {
    echo `echo "$@" | tr "[:lower:]" "[:upper:]"`
}

#----------------------------------------------------------------------------
# DE: Hauptmethode
# EN: main method
#----------------------------------------------------------------------------
case $1 in
    #----------------------------------------
    # DE: MACFilter starten
    # EN: start MACFilter
    #----------------------------------------
    'start')
        mf_status=`$0 "status"`
        case $mf_status in
            'stopped')
                add_rule filter macfilter_main "any any macfilter_list"
                mf_status=`$0 "status"`
                case $mf_status in
                    'started')
                        macfilter_log_info "start macfilter"
                        echo "started"
                    ;;
                    'stopped')  macfilter_echo_log_error "failed to start macfilter" ;;
                    *)          macfilter_echo_log_error "unexpected status returncode: '$mf_status'" ;;
                esac
            ;;
            'started')  macfilter_echo_log_warn "start macfilter failed: already running" ;;
            *)          macfilter_echo_log_error "unexpected status returncode: '$mf_status'" ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: MACFilter anhalten
    # EN: stop MACFilter
    #----------------------------------------
    'stop')
        mf_status=`$0 "status"`
        case $mf_status in
            'started')
                del_rule filter macfilter_main "any any macfilter_list"
                mf_status=`$0 "status"`
                case $mf_status in
                    'stopped')
                        macfilter_log_info "stopped macfilter"
                        echo "stopped"
                    ;;
                    'started')  macfilter_echo_log_error "failed to stop macfilter" ;;
                    *)          macfilter_echo_log_error "unexpected status returncode: '$mf_status'" ;;
                esac
            ;;
            'stopped')  macfilter_echo_log_warn "stopping macfilter failed: not running" ;;
            *)          macfilter_echo_log_error "unexpected status returncode: '$mf_status'" ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: Status von MACFilter ausgeben
    # EN: return state of MACfilter
    #----------------------------------------
    'status')
        if [ `iptables -nvL macfilter_list | grep "Chain" | cut -dr -f2 | cut -b8-` = "0" ]
        then
            echo "stopped"
        else
            echo "started"
        fi
    ;;
    
    
    #----------------------------------------
    # DE: MACFilter als Blacklist benutzen
    # EN: use MACFilter as blacklist
    #----------------------------------------
    'blacklist')
        mf_listtype=`$0 listtype`
        case $mf_listtype in
            'whitelist')
                macfilter_log_debug "set macfilter to blacklist"
                
                for mf_mac in `$0 "list"`
                do
                    mf_comment=`$0 "comment" "$mf_mac"`
                    macfilter_log_debug "set MAC '$mf_mac' to blacklist; comment: '$mf_comment'"
                    createIPTablesComment $mf_mac $mf_comment
                    del_rule filter macfilter_list "mac:$mf_mac any RETURN" "$mf_iptables_comment"
                    add_rule filter macfilter_list "mac:$mf_mac any $MACFILTER_BLOCKRULE" "$mf_iptables_comment"
                done
                
                del_rule filter macfilter_list "any any $MACFILTER_BLOCKRULE" "Whitelist"
                
                mf_listtype=`$0 "listtype"`
                case $mf_listtype in
                    'blacklist')
                        macfilter_log_info "macfilter set to blacklist"
                        echo "set to blacklist"
                    ;;
                    'whitelist')    macfilter_echo_log_error "failed to set macfilter to blacklist" ;;
                    *)              macfilter_echo_log_error "unexpected listtype returncode: '$mf_listtype'" ;;
                esac
            ;;
            'blacklist')    macfilter_echo_log_warn "set macfilter to blacklist failed: already running as blacklist" ;;
            *)              macfilter_echo_log_error "unexpected listtype returncode: '$mf_listtype'" ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: MACFilter als Whitelist benutzen
    # EN: use MACFilter as whitelist
    #----------------------------------------
    'whitelist')
        mf_listtype=`$0 "listtype"`
        case $mf_listtype in
            'blacklist')
                macfilter_log_debug "set macfilter to whitelist"
                
                for mf_mac in `$0 "list"`
                do
                    mf_comment=`$0 "comment" "$mf_mac"`
                    macfilter_log_debug "set MAC '$mf_mac' to whitelist; comment: '$mf_comment'"
                    createIPTablesComment $mf_mac $mf_comment
                    del_rule filter macfilter_list "mac:$mf_mac any $MACFILTER_BLOCKRULE" "$mf_iptables_comment"
                    add_rule filter macfilter_list "mac:$mf_mac any RETURN" "$mf_iptables_comment"
                done
                
                add_rule filter macfilter_list "any any $MACFILTER_BLOCKRULE" "Whitelist"
                
                mf_listtype=`$0 "listtype"`
                case $mf_listtype in
                    'whitelist')
                        macfilter_log_info "macfilter set to whitelist"
                        echo "set to whitelist"
                    ;;
                    'blacklist')    macfilter_echo_log_error "failed to set macfilter to whitelist" ;;
                    *)              macfilter_echo_log_error "unexpected listtype returncode: '$mf_listtype'" ;;
                esac
            ;;
            'whitelist')    macfilter_echo_log_warn "set macfilter to whitelist failed: already running as whitelist" ;;
            *)              macfilter_echo_log_error "unexpected listtype returncode: '$mf_listtype'" ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: List mit den MAC Adressen ausgeben
    # EN: return list with mac address
    #----------------------------------------
    'listtype')
        if [ `iptables -nvL macfilter_list | grep -c "* Whitelist *"` = 0 ]
        then
            echo "blacklist"
        else
            echo "whitelist"
        fi
    ;;
    
    
    #----------------------------------------
    # DE: MAC der Liste hinzufgen
    # EN: add MAC to list
    #----------------------------------------
    'add')
        mf_mac=`macfilter_make_uppercase $2`
        mf_comment=$3
        mf_exist=`$0 "exist" "$mf_mac"`
        case $mf_exist in
            'not found')
                mf_listtype=`$0 "listtype"`
                case $mf_listtype in
                    'blacklist')
                        # DE: MAC der Blacklist hinzufgen
                        # EN: add mac to blacklist
                        createIPTablesComment "$mf_mac" "$mf_comment"
                        add_rule filter macfilter_list "mac:$mf_mac any $MACFILTER_BLOCKRULE" "$mf_iptables_comment"
                        mf_exist=`$0 "exist" "$mf_mac"`
                        case $mf_exist in
                            'added')
                                echo "added"
                                macfilter_log_info "added MAC '$mf_mac' to blacklist, comment: '$mf_comment'"
                            ;;
                            'not found')    macfilter_echo_log_error "failed to add MAC '$mf_mac' to blacklist, comment: '$mf_comment'" ;;
                            *)              macfilter_echo_log_error "unexpected exist returncode: '$mf_exist'" ;;
                        esac
                    ;;
                    
                    'whitelist')
                        # DE: MAC der Whitelist hinzufgen
                        # EN: add mac to whitelist
                        createIPTablesComment "$mf_mac" "$mf_comment"
                        read mf_line tmp <<-EOF
`iptables -nvL macfilter_list --line-numbers | grep "Whitelist"`
EOF
                        ins_rule filter macfilter_list "mac:$mf_mac any RETURN" $mf_line "$mf_iptables_comment"
                        mf_exist=`$0 "exist" "$mf_mac"`
                        case $mf_exist in
                            'added')
                                echo "added"
                                macfilter_log_info "added MAC '$mf_mac' to whitelist, comment: '$mf_comment'"
                            ;;
                            'not found')    macfilter_echo_log_error "failed to add MAC '$mf_mac' to whitelist, comment: '$mf_comment'" ;;
                            *)              macfilter_echo_log_error "unexpected exist returncode: '$mf_exist'" ;;
                        esac
                    ;;
                    
                    *)  macfilter_echo_log_error "unexpected listtype returncode: '$mf_listtype'" ;;
                esac
            ;;
            'invalid mac')  macfilter_echo_log_warn "failed to add MAC '$mf_mac' to list: invalid mac" ;;
            'added')        macfilter_echo_log_warn "failed to add MAC '$mf_mac' to list: already added" ;;
            *)              macfilter_echo_log_error "unexpected exist returncode: '$mf_exist'" ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: Kontrolle ob eine MAC-Adresse schon in der Liste existiert
    # EN: check whether a mac already is in the list
    #----------------------------------------
    'exist')
        mf_mac=`macfilter_make_uppercase $2`
        REGEX_MACADDR="^[[:xdigit:]][[:xdigit:]](:[[:xdigit:]][[:xdigit:]]){5}$"
        if [ `echo "$mf_mac" | grep -cE "$REGEX_MACADDR"` != 0 ]
        then
            if [ `$0 "list" | grep -c "$2"` = 0 ]
            then
                echo "not found"
            else
                echo "added"
            fi
        else
            echo "invalid mac"
        fi
    ;;
    
    #----------------------------------------
    # DE: Kommentar einer MAC Adresse ausgeben
    # EN: return comment of a mac adress
    #----------------------------------------
    'comment')
        mf_mac=`macfilter_make_uppercase $2`
        mf_exist=`$0 "exist" "$mf_mac"`
        case $mf_exist in
            'added')
                mf_iptable=`iptables -nvL macfilter_list | grep "MAC:'$mf_mac'; comment:'"`
                if [ "$mf_iptable" != "" ]
                then
                    echo `expr "$mf_iptable" : ".*/\* MAC:'$mf_mac'; comment:'\(.*\)' \*/.*"`
                else
                    macfilter_echo_log_warn "failed to return comment: no entry in iptables found"
                fi
            ;;
            'invalid mac')  macfilter_echo_log_warn "failed to add MAC '$mf_mac' to list: invalid mac" ;;
            'not found')    macfilter_echo_log_warn "failed to return comment: MAC not found" ;;
            *)              macfilter_echo_log_error "unexpected exist returncode: '$mf_exist'" ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: MAC der Liste entfernen
    # EN: remove MAC to list
    #----------------------------------------
    'remove')
        mf_mac=`macfilter_make_uppercase $2`
        mf_comment=$3
        mf_exist=`$0 "exist" "$mf_mac"`
        case $mf_exist in
            'added')
                mf_listtype=`$0 "listtype"`
                mf_comment=`$0 "comment" "$mf_mac"`
                case $mf_listtype in
                    'blacklist')
                        # DE: MAC aus der Blacklist entfernen
                        # EN: remove mac from blacklist
                        createIPTablesComment "$mf_mac" "$mf_comment"
                        del_rule filter macfilter_list "mac:$mf_mac any $MACFILTER_BLOCKRULE" "$mf_iptables_comment"
                        mf_exist=`$0 "exist" "$mf_mac"`
                        case $mf_exist in
                            'not found')
                                echo "removed"
                                macfilter_log_info "removed MAC '$mf_mac' to blacklist, comment: '$mf_comment'"
                            ;;
                            'added')    macfilter_echo_log_error "failed to remove MAC '$mf_mac' from blacklist, comment: '$mf_comment'" ;;
                            *)          macfilter_echo_log_error "unexpected exist returncode: '$mf_exist'" ;;
                        esac
                    ;;
                    
                    'whitelist')
                        # DE: MAC aus der Whitelist entfernen
                        # EN: remove mac from whitelist
                        createIPTablesComment "$mf_mac" "$mf_comment"
                        del_rule filter macfilter_list "mac:$mf_mac any RETURN" "$mf_iptables_comment"
                        mf_exist=`$0 "exist" "$mf_mac"`
                        case $mf_exist in
                            'not found')
                                echo "removed"
                                macfilter_log_info "removed MAC '$mf_mac' to whitelist, comment: '$mf_comment'"
                            ;;
                            'added')    macfilter_echo_log_error "failed to remove MAC '$mf_mac' from whitelist, comment: '$mf_comment'" ;;
                            *)          
                            macfilter_echo_log_error "unexpected exist returncode: '$mf_exist'" ;;
                        esac
                    ;;
                    
                    *)  macfilter_echo_log_error "unexpected listtype returncode: '$mf_listtype'" ;;
                esac
                
            ;;
            'invalid mac')  macfilter_echo_log_warn "failed to add MAC '$mf_mac' to list: invalid mac" ;;
            'not found')    macfilter_echo_log_warn "failed to remove MAC '$mf_mac from list': MAC not found" ;;
            *)              macfilter_echo_log_error "unexpected exist returncode: '$mf_exist'" ;;
        esac
    ;;
    
    #----------------------------------------
    # DE: List mit den MAC Adressen ausgeben
    # EN: return list with mac address
    #----------------------------------------
    'list')
        echo `iptables -nvL macfilter_list | grep "MAC" | cut -dM -f2 | cut -b4-20`
    ;;
    
    
    #----------------------------------------
    # DE: ungltiger Parameter
    # EN: invalid Parameter
    #----------------------------------------
    'help'|'--help'|'-h')
        macfilter_log_debug "print help"
        cat <<-EOF
commands:
  start                 start macfilter
  stop                  stop macfilter
  status                status macfilter macfilter
  blacklist             set macfilter as blacklist
  whitelist             set macfilter as whitelist
  add <mac> <comment>   add the given mac address to the list
  exist <mac>           check whether the given mac adress exists in the list
  comment <mac>         returns the comment of the given mac adress
  remove <mac>          remove the given mac address frzm the list
  list                  list the mac addresses
  listtype              blacklist or whitelist
  help                  print this help
EOF
    ;;
    
    #----------------------------------------
    # DE: ungltiger Parameter
    # EN: invalid Parameter
    #----------------------------------------
    *)
        macfilter_log_error "unexpected parameter: '$1'"
        cat <<-EOF
Bad argument '$1'
Type '$0 help' for more information
EOF
    ;;
esac
#----------------------------------------------------------------------------
# macfilter.sh -- EOF ---
#----------------------------------------------------------------------------
